"""DO NOT MODIFY. Auto-generated by script/fingerprint_frontend."""

FINGERPRINTS = {
    "compatibility.js": "1686167ff210e001f063f5c606b2e74b",
    "core.js": "2a7d01e45187c7d4635da05065b5e54e",
    "frontend.html": "5a2a3d6181cc820f5b3e94d1a50def74",
    "mdi.html": "e91f61a039ed0a9936e7ee5360da3870",
    "micromarkdown-js.html": "93b5ec4016f0bba585521cf4d18dec1a",
    "panels/ha-panel-config.html": "ec48185c79000d0cfe5bbf38c7974944",
    "panels/ha-panel-dev-event.html": "d409e7ab537d9fe629126d122345279c",
    "panels/ha-panel-dev-info.html": "b0e55eb657fd75f21aba2426ac0cedc0",
    "panels/ha-panel-dev-mqtt.html": "94b222b013a98583842de3e72d5888c6",
    "panels/ha-panel-dev-service.html": "422b2c181ee0713fa31d45a64e605baf",
    "panels/ha-panel-dev-state.html": "7948d3dba058f31517d880df8ed0e857",
    "panels/ha-panel-dev-template.html": "f47b6910d8e4880e22cc508ca452f9b6",
    "panels/ha-panel-hassio.html": "b46e7619f3c355f872d5370741d89f6a",
    "panels/ha-panel-history.html": "fe2daac10a14f51fa3eb7d23978df1f7",
    "panels/ha-panel-iframe.html": "56930204d6e067a3d600cf030f4b34c8",
    "panels/ha-panel-kiosk.html": "b40aa5cb52dd7675bea744afcf9eebf8",
    "panels/ha-panel-logbook.html": "771afdcf48dc7e308b0282417d2e02d8",
    "panels/ha-panel-mailbox.html": "a8cca44ca36553e91565e3c894ea6323",
    "panels/ha-panel-map.html": "c2544fff3eedb487d44105cf94b335ec",
    "panels/ha-panel-shopping-list.html": "d8cfd0ecdb3aa6214c0f6908c34c7141"
}
